Enumeration 
  #JSONFile
  #MeshFile
  #ImageFile
EndEnumeration

Declare Init()
Declare SceneLoad(Scene.s)
Declare SceneLoadProgress()

Structure NewEntity
  id.i 
  ItemCreation.s
  
  mesh.i
  texture.i
  material.i
  
  meshBuffer.s
  meshFile.s
  meshFileExt.s
  meshLength.i  
  
  imageFile.s
  imageFileExt.s
  image.s
  imageBlend.b
  
  script.s  
  
  x.f
  y.f
  z.f
  rx.f
  ry.f
  rz.f
  sx.f
  sy.f
  sz.f  
EndStructure

Global NewList Work.NewEntity()
Global Cube.NewEntity
Global Sphere.NewEntity
Global Cylinder.NewEntity
Global Plane.NewEntity

Procedure Init()
  If FileSize("cache") = -1
    CreateDirectory("cache")
  EndIf 
  Add3DArchive("cache\", #PB_3DArchive_FileSystem ) 
  
  Cube\mesh = CreateCube(-1, 1)
  Sphere\mesh = CreateSphere(-1, 0.5)
  Cylinder\mesh = CreateCylinder(-1, 0.5, 1)
  Plane\mesh = CreatePlane(-1, 10, 10, 2, 2, 1, 1) 
EndProcedure

Procedure SceneLoad(Scene.s)
  Init()
  
  If LoadJSON(#JSONFile, Scene, #PB_JSON_NoCase)
    ExtractJSONList(JSONValue(#JSONFile), Work())
    ResetList(Work())
    SceneLoadProgress()
  EndIf
  ProcedureReturn ListSize(Work())
EndProcedure

Procedure SceneLoadProgress()
  Protected Entity
  Protected *InputBuffer
  Protected *OutputBuffer
  Protected ImageFile.s, Image, ImagePlugin, Mesh
  Static CurrentElement
  Protected CountElements = ListSize(Work())
  
  If CurrentElement <> CountElements
    NextElement(Work())
    CurrentElement + 1
    With Work()
      Select \ItemCreation 
        Case "Mesh"
          *InputBuffer = AllocateMemory(\meshLength * 1.35)
          Base64Decoder(@\meshBuffer, Len(\meshBuffer), *InputBuffer, \meshLength)
          
          If CreateFile(#MeshFile, "cache\" + \meshFile) <> 0
            WriteData(#MeshFile, *InputBuffer, \meshLength)
            CloseFile(#MeshFile)
          EndIf
          
          Mesh = LoadMesh(#PB_Any, \meshFile)
          If Mesh
            BuildMeshShadowVolume(Mesh)
            DeleteFile("cache\" + \meshFile)
            \id = CreateEntity(-1, MeshID(Mesh), #PB_Material_None, \x, \y, \z)
          EndIf
          EntityPhysicBody(\id, #PB_Entity_StaticBody)
          
        Case "Cube"
          \id = CreateEntity(-1, MeshID(Cube\mesh), #PB_Material_None, \x, \y, \z)
          EntityPhysicBody(\id, #PB_Entity_StaticBody)
        Case "Sphere"
          \id = CreateEntity(-1, MeshID(Sphere\mesh), #PB_Material_None, \x, \y, \z)
          EntityPhysicBody(\id, #PB_Entity_StaticBody)
        Case "Cylinder"
          \id = CreateEntity(-1, MeshID(Cylinder\mesh), #PB_Material_None, \x, \y, \z)
          EntityPhysicBody(\id, #PB_Entity_StaticBody)
        Case "Plane"
          \id = CreateEntity(-1, MeshID(Plane\mesh), #PB_Material_None, \x, \y, \z)
          EntityPhysicBody(\id, #PB_Entity_StaticBody)
      EndSelect 
      
      RotateEntity(\id, \rx, \ry, \rz)      
      ScaleEntity(\id, \sx, \sy, \sz)      
      
      If \image <> ""       
        *InputBuffer = AllocateMemory(Len(\image)*1.33)
        Base64Decoder(@\image, Len(\image), *InputBuffer, Len(\image)) 
        
        If CatchImage(Image, *InputBuffer)
          Select \ImageFileExt   
            Case "bmp" : ImagePlugin = #PB_ImagePlugin_BMP  
            Case "png" : ImagePlugin = #PB_ImagePlugin_PNG
            Case "jpg" : ImagePlugin = #PB_ImagePlugin_JPEG    
          EndSelect
          
          SaveImage(Image, "cache\" + \ImageFile, ImagePlugin) 
          
          \texture = LoadTexture(#PB_Any, \ImageFile)          
          \material = CreateMaterial(#PB_Any, TextureID(\texture))
          DeleteFile("cache\" + \ImageFile)
          If \imageBlend   
            MaterialBlendingMode(\material, #PB_Material_AlphaBlend)
          EndIf
          
          SetEntityMaterial(\id, MaterialID(\material))
        EndIf           
      EndIf
    EndWith
    If *InputBuffer
      FreeMemory(*InputBuffer)
    EndIf
    ProcedureReturn Int((CurrentElement/CountElements) * 100)
  Else
    DeleteDirectory("cache", "*.*", #PB_FileSystem_Force)
    ProcedureReturn #False
  EndIf
EndProcedure
; IDE Options = PureBasic 5.31 (Windows - x86)
; CursorPosition = 146
; FirstLine = 93
; Folding = ---
; EnableXP